@@ -145,9 +145,7 @@ This is just a zip file beginning with the magic string "C<PK\003\004>".
 
 The pre-computed cache name.  A pack('Z40') string of the value of -T 
 (--tempcache) or the hash of the file, followed by C<\0CACHE>.  The hash
-of the file is calculated with L<Digest::SHA>, L<Digest::SHA1>, or 
-L<Digest::MD5>.  If none of those modules is available, the C<mtime> of
-the file is used.
+of the file is calculated with L<Digest::SHA>.
 
 A pack('N') number of the total length of FILE and PAR sections,
 followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
@@ -156,7 +154,8 @@ followed by a 8-bytes magic string: "C<\012PAR.pm\012>".
 
 =cut
 
-my ($par_temp, $progname, @tmpfile);
+
+my ($PAR_MAGIC, $par_temp, $progname, @tmpfile);
 END { if ($ENV{PAR_CLEAN}) {
     require File::Temp;
     require File::Basename;
@@ -205,6 +204,7 @@ rm '" . $tmp->filename . "'
 
 BEGIN {
     Internals::PAR::BOOT() if defined &Internals::PAR::BOOT;
+    $PAR_MAGIC = "\nPAR.pm\n";
 
     eval {
 
@@ -234,33 +234,41 @@ my ($start_pos, $data_pos);
     # Search for the "\nPAR.pm\n signature backward from the end of the file
     my $buf;
     my $size = -s $progname;
-    my $offset = 512;
-    my $idx = -1;
-    while (1)
-    {
-        $offset = $size if $offset > $size;
-        seek _FH, -$offset, 2 or die qq[seek failed on "$progname": $!];
-        my $nread = read _FH, $buf, $offset;
-        die qq[read failed on "$progname": $!] unless $nread == $offset;
-        $idx = rindex($buf, "\nPAR.pm\n");
-        last if $idx >= 0 || $offset == $size || $offset > 128 * 1024;
-        $offset *= 2;
+    my $chunk_size = 64 * 1024;
+    my $magic_pos;
+
+    if ($size <= $chunk_size) {
+        $magic_pos = 0;
+    } elsif ((my $m = $size % $chunk_size) > 0) {
+        $magic_pos = $size - $m;
+    } else {
+        $magic_pos = $size - $chunk_size;
     }
-    last unless $idx >= 0;
+    # in any case, $magic_pos is a multiple of $chunk_size
+
+    while ($magic_pos >= 0) {
+        seek(_FH, $magic_pos, 0);
+        read(_FH, $buf, $chunk_size + length($PAR_MAGIC));
+        if ((my $i = rindex($buf, $PAR_MAGIC)) >= 0) {
+            $magic_pos += $i;
+            last;
+        }
+        $magic_pos -= $chunk_size;
+    }
+    last if $magic_pos < 0;
 
     # Seek 4 bytes backward from the signature to get the offset of the 
     # first embedded FILE, then seek to it
-    $offset -= $idx - 4;
-    seek _FH, -$offset, 2;
-    read _FH, $buf, 4;
-    seek _FH, -$offset - unpack("N", $buf), 2;
+    seek _FH, $magic_pos - 4, 0;
     read _FH, $buf, 4;
+    seek _FH, $magic_pos - 4 - unpack("N", $buf), 0;
+    $data_pos = tell _FH;
 
-    $data_pos = (tell _FH) - 4;
     # }}}
 
     # Extracting each file into memory {{{
     my %require_list;
+    read _FH, $buf, 4;                           # read the first "FILE"
     while ($buf eq "FILE") {
         read _FH, $buf, 4;
         read _FH, $buf, unpack("N", $buf);
@@ -299,26 +307,19 @@ my ($start_pos, $data_pos);
 
         return if ref $module or !$module;
 
-        my $filename = delete $require_list{$module} || do {
-            my $key;
-            foreach (keys %require_list) {
-                next unless /\Q$module\E$/;
-                $key = $_; last;
-            }
-            delete $require_list{$key} if defined($key);
-        } or return;
+        my $info = delete $require_list{$module} or return;
 
-        $INC{$module} = "/loader/$filename/$module";
+        $INC{$module} = "/loader/$info/$module";
 
         if ($ENV{PAR_CLEAN} and defined(&IO::File::new)) {
             my $fh = IO::File->new_tmpfile or die $!;
             binmode($fh);
-            print $fh $filename->{buf};
+            print $fh $info->{buf};
             seek($fh, 0, 0);
             return $fh;
         }
         else {
-            my $filename = _tempfile("$filename->{crc}.pm", $filename->{buf});
+            my $filename = _tempfile("$info->{crc}.pm", $info->{buf});
 
             open my $fh, '<', $filename or die "can't read $filename: $!";
             binmode($fh);
@@ -359,7 +360,7 @@ my ($start_pos, $data_pos);
     # }}}
 
     last unless $buf eq "PK\003\004";
-    $start_pos = (tell _FH) - 4;
+    $start_pos = (tell _FH) - 4;                # start of zip
 }
 # }}}
 
@@ -447,6 +448,7 @@ if ($out) {
         #local $INC{'Cwd.pm'} = __FILE__ if $^O ne 'MSWin32';
         require IO::File;
         require Archive::Zip;
+        require Digest::SHA;
     }
 
     my $par = shift(@ARGV);
@@ -454,10 +456,6 @@ if ($out) {
 
 
     if (defined $par) {
-        # increase the chunk size for Archive::Zip so that it will find the EOCD
-        # even if more stuff has been appended to the .par
-        Archive::Zip::setChunkSize(128*1024);
-
         open my $fh, '<', $par or die "Cannot find '$par': $!";
         binmode($fh);
         bless($fh, 'IO::File');
@@ -518,6 +516,7 @@ if ($out) {
     if ($bundle) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
+
         init_inc();
 
         require_modules();
@@ -602,30 +601,19 @@ if ($out) {
 
     $cache_name = substr $cache_name, 0, 40;
     if (!$cache_name and my $mtime = (stat($out))[9]) {
-        my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
-            || eval { require Digest::SHA1; Digest::SHA1->new }
-            || eval { require Digest::MD5; Digest::MD5->new };
+        my $ctx = Digest::SHA->new(1);
+        open(my $fh, "<", $out);
+        binmode($fh);
+        $ctx->addfile($fh);
+        close($fh);
 
-        # Workaround for bug in Digest::SHA 5.38 and 5.39
-        my $sha_version = eval { $Digest::SHA::VERSION } || 0;
-        if ($sha_version eq '5.38' or $sha_version eq '5.39') {
-            $ctx->addfile($out, "b") if ($ctx);
-        }
-        else {
-            if ($ctx and open(my $fh, "<$out")) {
-                binmode($fh);
-                $ctx->addfile($fh);
-                close($fh);
-            }
-        }
-
-        $cache_name = $ctx ? $ctx->hexdigest : $mtime;
+        $cache_name = $ctx->hexdigest;
     }
     $cache_name .= "\0" x (41 - length $cache_name);
     $cache_name .= "CACHE";
     $fh->print($cache_name);
     $fh->print(pack('N', $fh->tell - length($loader)));
-    $fh->print("\nPAR.pm\n");
+    $fh->print($PAR_MAGIC);
     $fh->close;
     chmod 0755, $out;
     # }}}
@@ -650,10 +638,16 @@ if ($out) {
         require File::Find;
         require Archive::Zip;
     }
-    my $zip = Archive::Zip->new;
-    my $fh = IO::File->new;
+
+    my $fh = IO::File->new;                             # Archive::Zip operates on an IO::Handle
     $fh->fdopen(fileno(_FH), 'r') or die "$!: $@";
+
+    # Temporarily increase the chunk size for Archive::Zip so that it will find the EOCD
+    # even if lots of stuff has been appended to the pp'ed exe (e.g. by OSX codesign).
+    Archive::Zip::setChunkSize(-s _FH);
+    my $zip = Archive::Zip->new;
     $zip->readFromFileHandle($fh, $progname) == Archive::Zip::AZ_OK() or die "$!: $@";
+    Archive::Zip::setChunkSize(64 * 1024);
 
     push @PAR::LibCache, $zip;
     $PAR::LibCache{$progname} = $zip;
@@ -732,6 +726,7 @@ sub require_modules {
     require IO::File;
     require Compress::Zlib;
     require Archive::Zip;
+    require Digest::SHA;
     require PAR;
     require PAR::Heavy;
     require PAR::Dist;
@@ -789,24 +784,18 @@ sub _set_par_temp {
                 $stmpdir .= "$Config{_delim}cache-" . $buf;
             }
             else {
-                my $ctx = eval { require Digest::SHA; Digest::SHA->new(1) }
-                    || eval { require Digest::SHA1; Digest::SHA1->new }
-                    || eval { require Digest::MD5; Digest::MD5->new };
+                my $digest = eval 
+                {
+                    require Digest::SHA; 
+                    my $ctx = Digest::SHA->new(1);
+                    open(my $fh, "<", $progname);
+                    binmode($fh);
+                    $ctx->addfile($fh);
+                    close($fh);
+                    $ctx->hexdigest;
+                } // $mtime;
 
-                # Workaround for bug in Digest::SHA 5.38 and 5.39
-                my $sha_version = eval { $Digest::SHA::VERSION } || 0;
-                if ($sha_version eq '5.38' or $sha_version eq '5.39') {
-                    $ctx->addfile($progname, "b") if ($ctx);
-                }
-                else {
-                    if ($ctx and open(my $fh, "<$progname")) {
-                        binmode($fh);
-                        $ctx->addfile($fh);
-                        close($fh);
-                    }
-                }
-
-                $stmpdir .= "$Config{_delim}cache-" . ( $ctx ? $ctx->hexdigest : $mtime );
+                $stmpdir .= "$Config{_delim}cache-$digest"; 
             }
             close($fh);
         }
@@ -906,7 +895,7 @@ sub _par_init_env {
         $ENV{PAR_INITIALIZED} = 2;
     }
 
-    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ARGC ARGV_0 ) ) {
+    for (qw( SPAWNED TEMP CLEAN DEBUG CACHE PROGNAME ) ) {
         delete $ENV{'PAR_'.$_};
     }
     for (qw/ TMPDIR TEMP CLEAN DEBUG /) {
